/// <reference lib="dom" />
import type { KeysOfType, WritableKeys } from "./general";
export * as JSX from "./jsx";
export declare const Fragment: unique symbol;
/**
 * Used to inject HTML directly into the document.
 */
export declare function Raw(_props: {
    html: string;
}): null;
export declare type Children = Element | string | number | null | undefined | Children[];
export declare type Component<P> = (props: P) => Element | null | undefined;
declare type BannedElementKeys = "dataset" | "innerHTML" | "outerHTML" | "innerHTML" | "innerText" | "textContent" | "style";
interface RemapKeys {
    className: "class";
    htmlFor: "for";
    httpEquiv: "http-equiv";
}
declare type ElementKeys<T> = Exclude<WritableKeys<T>, BannedElementKeys | KeysOfType<T, Function>>;
declare type BasicHtmlElements = {
    [K in keyof HTMLElementTagNameMap]: {
        [K2 in ElementKeys<HTMLElementTagNameMap[K]> as K2 extends keyof RemapKeys ? RemapKeys[K2] : K2]?: K2 extends "children" ? Children : HTMLElementTagNameMap[K][K2];
    } & {
        style?: string;
    } & (K extends "meta" ? {
        charSet?: string;
    } : unknown);
};
export interface IntrinsicElements extends BasicHtmlElements {
}
export interface Element {
    tag: typeof Fragment | keyof IntrinsicElements | Component<any>;
    props: object | null;
    children: Children[];
}
/**
 * JSX factory function to create an "element" that can later be rendered with {@link renderElement}
 * @param tag
 * @param props
 * @param children
 */
export declare function createElement(tag: typeof Fragment | keyof IntrinsicElements | Component<any>, props: object | null, ...children: Children[]): Element;
export declare function renderElement(element: Element | null | undefined): string;
