"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findElementById = exports.renderElement = exports.createElement = exports.JSX = exports.Fragment = exports.Raw = void 0;
var jsx_1 = require("./jsx");
Object.defineProperty(exports, "Raw", { enumerable: true, get: function () { return jsx_1.Raw; } });
Object.defineProperty(exports, "Fragment", { enumerable: true, get: function () { return jsx_1.Fragment; } });
exports.JSX = require("./jsx");
const jsx = require("./jsx");
function escapeHtml(html) {
    return html.replace(/[&<>'"]/g, (c) => ({
        "&": "&amp;",
        "<": "&lt;",
        ">": "&gt;",
        '"': "&quot;",
        "'": "&#39;",
    }[c]));
}
const voidElements = new Set([
    "area",
    "base",
    "br",
    "col",
    "embed",
    "hr",
    "img",
    "input",
    "link",
    "meta",
    "param",
    "source",
    "track",
    "wbr",
]);
/**
 * JSX factory function to create an "element" that can later be rendered with {@link renderElement}
 * @param tag
 * @param props
 * @param children
 */
function createElement(tag, props, ...children) {
    return { tag, props, children };
}
exports.createElement = createElement;
function renderElement(element) {
    if (!element) {
        return "";
    }
    const { tag, props, children } = element;
    if (typeof tag === "function") {
        if (tag === jsx.Raw) {
            return String(props.html);
        }
        return renderElement(tag(Object.assign({ children }, props)));
    }
    const html = [];
    if (tag !== jsx.Fragment) {
        html.push("<", tag);
        for (const [key, val] of Object.entries(props !== null && props !== void 0 ? props : {})) {
            if (val == null)
                continue;
            if (typeof val == "boolean") {
                if (val) {
                    html.push(" ", key);
                }
            }
            else {
                html.push(" ", key, "=", JSON.stringify(val));
            }
        }
    }
    let hasChildren = false;
    if (children.length) {
        hasChildren = true;
        if (tag !== jsx.Fragment)
            html.push(">");
        renderChildren(children);
    }
    if (tag !== jsx.Fragment) {
        if (!hasChildren) {
            if (voidElements.has(tag)) {
                html.push("/>");
            }
            else {
                html.push("></", tag, ">");
            }
        }
        else {
            html.push("</", tag, ">");
        }
    }
    return html.join("");
    function renderChildren(children) {
        for (const child of children) {
            if (!child)
                continue;
            if (Array.isArray(child)) {
                renderChildren(child);
            }
            else if (typeof child === "string" || typeof child === "number") {
                html.push(escapeHtml(child.toString()));
            }
            else {
                html.push(renderElement(child));
            }
        }
    }
}
exports.renderElement = renderElement;
/**
 * Extremely limited query function for reaching into a {@link JSX} structure
 * to find an element that's already been declared. This can be used to inject content into a page
 * after it has been created in the `page.end` hook.
 */
function findElementById(root, id) {
    const checks = [];
    let current = root;
    do {
        if (current.props && current.props.id === id) {
            return current;
        }
        addChildren(current.children);
    } while ((current = checks.shift()));
    function addChildren(children) {
        for (const child of children) {
            if (Array.isArray(child)) {
                addChildren(child);
            }
            else if (child && typeof child === "object") {
                checks.push(child);
            }
        }
    }
}
exports.findElementById = findElementById;
//# sourceMappingURL=template.js.map